@@ -30,8 +30,9 @@
 #'@return A list with the following elements:\itemize{
 #'   \item \code{score}: approximation of the set observed score
 #'   \item \code{q}: observation-level contributions to the score
-#'   \item \code{q_ext}: pseudo-observations used to compute covariance taking into account the contributions of OLS estimates
-#'   \item \code{gene_scores}: approximation of the individual gene scores
+#'   \item \code{q_ext}: pseudo-observations used to compute the covariance,
+#'    taking into account the contributions of OLS estimates
+#'   \item \code{gene_scores_unscaled}: a vector of the approximations of the individual gene scores
 #' }
 #'
 #'@seealso \code{\link[CompQuadForm]{davies}}
@@ -44,7 +45,7 @@
 #'########################
 #'n <- 100
 #'r <- 12
-#'t <- matrix(rep(1:3), 4, ncol=1, nrow=r)
+#'t <- matrix(rep(1:3), r/3, ncol=1, nrow=r)
 #'sigma <- 0.4
 #'b0 <- 1
 #'
@@ -59,10 +60,11 @@
 #'
 #'#run test
 #'scoreTest <- vc_score(y, x, phi=t, w=matrix(1, ncol=ncol(y), nrow=nrow(y)),
-#'                     Sigma_xi=matrix(1), indiv=rep(1:4, each=3))
+#'                     Sigma_xi=matrix(1), indiv=rep(1:(r/3), each=3))
 #'scoreTest$score
 #'
 #'@importFrom CompQuadForm davies
+#'@importFrom stats model.matrix
 #'
 #'@export
 vc_score <- function(y, x, indiv, phi, w, Sigma_xi = diag(ncol(phi)), na_rm = FALSE) {
@@ -72,7 +74,6 @@ vc_score <- function(y, x, indiv, phi, w, Sigma_xi = diag(ncol(phi)), na_rm = FA
   }
 
   ## dimensions check------
-# browser()
   stopifnot(is.matrix(y))
   stopifnot(is.matrix(x))
   stopifnot(is.matrix(phi))
@@ -105,7 +106,7 @@ vc_score <- function(y, x, indiv, phi, w, Sigma_xi = diag(ncol(phi)), na_rm = FA
 
   ## OLS for conditional mean -----
   y_T <- t(y)
-  if(na_rm){
+  if(na_rm & sum(is.na(y_T))>0){
     y_T0 <- y_T
     y_T0[is.na(y_T0)] <- 0
     yt_mu <- y_T - x%*%solve(crossprod(x))%*%t(x)%*%y_T0
@@ -166,7 +167,7 @@ vc_score <- function(y, x, indiv, phi, w, Sigma_xi = diag(ncol(phi)), na_rm = FA
   sig_eps_inv_T <- t(w)
   phi_sig_xi_sqrt <- phi%*%sig_xi_sqrt
 
-  T_fast <- do.call(cbind, replicate(K, sig_eps_inv_T, simplify = FALSE))*t(matrix(rep(t(phi_sig_xi_sqrt), each=g), nrow=g*K))
+  T_fast <- do.call(cbind, replicate(K, sig_eps_inv_T, simplify = FALSE))*matrix(apply(phi_sig_xi_sqrt, 2, rep, g), ncol = g*K)
   ###---------------------
   ## the structure of T_fast is time_basis_1*gene_1, time_basis_1*gene_2, ...,
   ## time_basis_1*gene_p, ..., time_basis_K*gene_1, ..., time_basis_K*gene_p
@@ -183,25 +184,40 @@ vc_score <- function(y, x, indiv, phi, w, Sigma_xi = diag(ncol(phi)), na_rm = FA
   #m_dt <- data.table("indiv"=factor(rep(c(1:20), each=5)), mbig)
   #temp <- m_dt[, lapply(.SD, sum), by=indiv]
 
-  #those 2 by statements represent the longest AND more memory intensive part of this for genewise analysis:
-  q <- do.call(rbind, by(q_fast, indiv, FUN=colSums, simplify=FALSE, na.rm = na_rm))
-  XT_fast <- t(x)%*%T_fast/nb_indiv
-  avg_xtx_inv_tx <- nb_indiv*solve(t(x)%*%x)%*%t(x)
-  U_XT <- matrix(yt_mu, ncol=g*n_t, nrow=n)*t(avg_xtx_inv_tx)%*%XT_fast
-  U_XT_indiv <- do.call(rbind, by(U_XT, indiv, FUN=colSums, simplify=FALSE, na.rm = na_rm))
+  #the 2 by statements below used to represent the longest AND most memory intensive part of this for genewise analysis:
+
+  if(length(levels(indiv))>1){
+    indiv_mat <- stats::model.matrix(~0 + factor(indiv))
+  }else{
+    indiv_mat <- matrix(as.numeric(indiv), ncol=1)
+  }
+
+  if(na_rm & sum(is.na(q_fast))>0){
+    q_fast[is.na(q_fast)] <- 0
+  }
+  q <- crossprod(indiv_mat, q_fast)
+  XT_fast <- crossprod(x, T_fast)/nb_indiv
+  avg_xtx_inv_tx <- nb_indiv*tcrossprod(solve(crossprod(x, x)), x)
+  U_XT <- matrix(yt_mu, ncol=g*n_t, nrow=n)*crossprod(avg_xtx_inv_tx, XT_fast)
+  if(na_rm & sum(is.na(U_XT))>0){
+    U_XT[is.na(U_XT)] <- 0
+  }
+  U_XT_indiv <- crossprod(indiv_mat, U_XT)
   q_ext <-  q - U_XT_indiv
   #sapply(1:6, function(i){(q_ext[i,] - q_ext_fast_indiv[i,])})
 
 
-  gene_inds <- lapply(1:g, function(x){x + (g)*(0:(K-1))})
 
   qq <- colSums(q, na.rm = na_rm)^2/nb_indiv # genewise scores
-  gene_Q <- sapply(gene_inds, function(x) sum(qq[x])) # to be optimized
-  #do.call(rbind, by(t(qq), rep(1:K, g), FUN=colSums, simplify=FALSE))
+
+  #unlist(by(data=matrix(qq, ncol=1), INDICES=rep(1:g, K), FUN=sum, simplify = FALSE)) # veryslow
+  #gene_inds <- lapply(1:g, function(x){x + (g)*(0:(K-1))})
+  #gene_Q <- sapply(gene_inds, function(x) sum(qq[x])) # old computation
+  #gene_Q <- tcrossprod(qq, matrix(diag(g), nrow=g, ncol=g*K))[1, ] # faster
+  gene_Q <- rowSums(matrix(qq, ncol=K)) # even faster
 
   QQ <- sum(qq) #nb_indiv=nrow(q) # set score
 
   return(list("score"=QQ, "q" = q, "q_ext"=q_ext,
-              "gene_scores_unscaled" = gene_Q,
-              "gene_inds" = gene_inds))
+              "gene_scores_unscaled" = gene_Q))
 }
